@@ -26,7 +26,9 @@
  */
 
 #include <config.h>
+#include <string.h>
 #include <gtk/gtksignal.h>
+
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-loader.h"
 #include "gdk-pixbuf-io.h"
@@ -271,17 +273,17 @@ gdk_pixbuf_loader_frame_done (GdkPixbufFrame *frame, gpointer loader)
 		priv->animation = g_new0 (GdkPixbufAnimation, 1);
 		priv->animation->n_frames = 0;
 		priv->animation->ref_count = 1;
-		priv->animation->width  = gdk_pixbuf_get_width  (frame->pixbuf);
-		priv->animation->height = gdk_pixbuf_get_height (frame->pixbuf);
+		priv->animation->width  = gdk_pixbuf_get_width  (frame->pixbuf) + frame->x_offset;
+		priv->animation->height = gdk_pixbuf_get_height (frame->pixbuf) + frame->y_offset;
 	} else {
 		int w, h;
 
 		/* update bbox size */
-		w = gdk_pixbuf_get_width (frame->pixbuf);
-		h = gdk_pixbuf_get_height (frame->pixbuf);
+		w = gdk_pixbuf_get_width (frame->pixbuf) + frame->x_offset;
+		h = gdk_pixbuf_get_height (frame->pixbuf) + frame->y_offset;
 
 		if (w > priv->animation->width) {
-			priv->animation->width = h;
+			priv->animation->width = w;
 		}
 		if (h > priv->animation->height) {
 			priv->animation->height = h;
@@ -298,6 +300,32 @@ gdk_pixbuf_loader_frame_done (GdkPixbufFrame *frame, gpointer loader)
 static void
 gdk_pixbuf_loader_animation_done (GdkPixbuf *pixbuf, gpointer loader)
 {
+	GdkPixbufLoaderPrivate *priv = NULL;
+	GdkPixbufFrame    *frame;
+	GList *current = NULL;
+	gint h, w;
+
+	priv = GDK_PIXBUF_LOADER (loader)->private;
+	priv->pixbuf = NULL;
+
+	current = gdk_pixbuf_animation_get_frames (priv->animation);
+
+	while (current) {
+		frame = (GdkPixbufFrame *) current->data;
+
+		/* update bbox size */
+		w = gdk_pixbuf_get_width (frame->pixbuf) + frame->x_offset;
+		h = gdk_pixbuf_get_height (frame->pixbuf) + frame->y_offset;
+
+		if (w > priv->animation->width) {
+			priv->animation->width = w;
+		}
+		if (h > priv->animation->height) {
+			priv->animation->height = h;
+		}
+		current = current->next;
+	}
+
 	gtk_signal_emit (GTK_OBJECT (loader),
 			 pixbuf_loader_signals[ANIMATION_DONE]);
 }
@@ -433,7 +461,8 @@ gdk_pixbuf_loader_write (GdkPixbufLoader *loader, const guchar *buf, size_t coun
  * the size of the image that will be allocated.  If the loader has not received
  * enough data via gdk_pixbuf_loader_write(), then this function returns NULL.
  * The returned pixbuf will be the same in all future calls to the loader, so
- * simply calling gdk_pixbuf_ref() should be sufficient to continue using it.
+ * simply calling gdk_pixbuf_ref() should be sufficient to continue using it.  Additionally,
+ * if the loader is an animation, it will return the first frame of the animation.
  *
  * Return value: The GdkPixbuf that the loader is creating, or NULL if not
  * enough data has been read to determine how to create the image buffer.
@@ -448,6 +477,16 @@ gdk_pixbuf_loader_get_pixbuf (GdkPixbufLoader *loader)
 
 	priv = loader->private;
 
+	if (priv->animation) {
+		GList *list;
+		
+		list = gdk_pixbuf_animation_get_frames (priv->animation);
+		if (list != NULL) {
+			GdkPixbufFrame *frame = list->data;
+
+			return gdk_pixbuf_frame_get_pixbuf (frame);
+		}
+	}
 	return priv->pixbuf;
 }
 
