@@ -30,7 +30,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <tiffio.h>
-#include "gdk-pixbuf.h"
+#include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
 
@@ -68,7 +68,7 @@ gdk_pixbuf__tiff_image_load_real (FILE *f, TiffData *context)
 	TIFFGetField (tiff, TIFFTAG_IMAGEWIDTH, &w);
 	TIFFGetField (tiff, TIFFTAG_IMAGELENGTH, &h);
 	num_pixs = w * h;
-	pixbuf = gdk_pixbuf_new (ART_PIX_RGB, TRUE, 8, w, h);
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, w, h);
 
 	if (context)
 		(* context->prepare_func) (pixbuf, context->user_data);
@@ -155,12 +155,14 @@ gdk_pixbuf__tiff_image_begin_load (ModulePreparedNotifyFunc prepare_func,
 	context->tempname = g_strdup ("/tmp/gdkpixbuf-tif-tmp.XXXXXX");
 	fd = mkstemp (context->tempname);
 	if (fd < 0) {
+	        g_free (context->tempname);
 		g_free (context);
 		return NULL;
 	}
 
 	context->file = fdopen (fd, "w");
 	if (context->file == NULL) {
+		g_free (context->tempname);
 		g_free (context);
 		return NULL;
 	}
@@ -182,6 +184,7 @@ gdk_pixbuf__tiff_image_stop_load (gpointer data)
 
 	fclose (context->file);
 	unlink (context->tempname);
+	g_free (context->tempname);
 	g_free ((TiffData *) context);
 }
 
@@ -200,3 +203,8 @@ gdk_pixbuf__tiff_image_load_increment (gpointer data, guchar *buf, guint size)
 	return TRUE;
 }
 
+
+
+
+
+
